﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Cors;
using Microsoft.AspNetCore.Mvc;
using Newtonsoft.Json;
using Syncfusion.Reporting.Web;
using Syncfusion.Reporting.Web.ReportViewer;

// For more information on enabling Web API for empty projects, visit https://go.microsoft.com/fwlink/?LinkID=397860

namespace SMOApi.Controllers
{
    class Customer
    {
        public string Name { get; set; }
        public int Age { get; set; }
    };

    [Route("api/[controller]/[action]")]
    public class ReportController : Controller, IReportController
    {
        // Report viewer requires a memory cache to store the information of consecutive client request and
        // have the rendered Report Viewer information in server.
        private Microsoft.Extensions.Caching.Memory.IMemoryCache _cache;

        // IHostingEnvironment used with sample to get the application data from wwwroot.
        private Microsoft.AspNetCore.Hosting.IHostingEnvironment _hostingEnvironment;

        // Post action to process the report from server based json parameters and send the result back to the client.
        public ReportController(Microsoft.Extensions.Caching.Memory.IMemoryCache memoryCache,
            Microsoft.AspNetCore.Hosting.IHostingEnvironment hostingEnvironment
            )
        {
            _cache = memoryCache;
            _hostingEnvironment = hostingEnvironment;
            
        }

        // Post action to process the report from server based json parameters and send the result back to the client.
        [HttpPost]
        public object PostReportAction([FromBody] Dictionary<string, object> jsonArray)
        {
            return ReportHelper.ProcessReport(jsonArray, this, this._cache);
        }

        // Method will be called to initialize the report information to load the report with ReportHelper for processing.
        public void OnInitReportOptions(ReportViewerOptions reportOption)
        {
            string basePath = _hostingEnvironment.WebRootPath;
            // Here, we have loaded the sales-order-detail.rdl report from application the folder wwwroot\Resources. sales-order-detail.rdl should be there in wwwroot\Resources application folder.
            FileStream reportStream = new FileStream(basePath + @"\reports\"
                + reportOption.ReportModel.ReportPath, FileMode.Open, FileAccess.Read);
            reportOption.ReportModel.Stream = reportStream;

            IList<Customer> customers = new List<Customer>() { new Customer() { Name = "Name1", Age = 25},
                new Customer() { Name = "Name2", Age = 22},
                new Customer() { Name = "Name3", Age = 23}, new Customer() { Name = "Name4", Age = 26}
            };

            reportOption.ReportModel.DataSources.Add(new ReportDataSource { Name = "DSTCustomers", Value = customers });
            
            string resourceName = "ReportTests.Resources.Arial.ttf";

            if (reportOption.ReportModel.PDFOptions == null)
            {
                reportOption.ReportModel.PDFOptions = new Syncfusion.Reporting.Writer.PDFOptions();
            }

            if (reportOption.ReportModel.PDFOptions.Fonts == null)
            {
                reportOption.ReportModel.PDFOptions.Fonts =
                    new Dictionary<string, Stream>(StringComparer.OrdinalIgnoreCase);
            }
            
            reportOption.ReportModel.PDFOptions.Fonts.Add("Arial",
                Assembly.GetExecutingAssembly().GetManifestResourceStream(resourceName));
        }

        // Method will be called when reported is loaded with internally to start to layout process with ReportHelper.
        public void OnReportLoaded(ReportViewerOptions reportOption)
        {
        }

        //Get action for getting resources from the report
        [ActionName("GetResource")]
        [AcceptVerbs("GET")]
        // Method will be called from Report Viewer client to get the image src for Image report item.
        public object GetResource(ReportResource resource)
        {
            return ReportHelper.GetResource(resource, this, _cache);
        }

        [HttpPost]
        public object PostFormReportAction()
        {  
            return ReportHelper.ProcessReport(null, this, _cache);
        }

    }



}
